#include "BSP/TIMER/timer.h"


//------------------------------  ------------------------------
u16 Tcount;
bit B_1T;
bit B_10T;
bit B_100T;
bit B_1000T;

//------------------------------  ------------------------------
/**
 * @name Timer0_Init
 * @brief ʱ/0
 * @param Mode:     ģʽ    0: 16λԶװ, 1: 16λʱ/, 2: 8λԶװ, 3: 16λԶװ
 * @param ClkSource ʱԴ      0: 12T          1: 1T    (12TģʽԶʱ)
 * @param Timer     ʱʱ    0-65535
 * @param MsUs      ʱ׼    0: ms   1: us
 * @version 汾v1.0
 * @date ڣ2025-4-19
 * @author ߣϫ
 * @note עͣ
 *      TIM_Value   װسֵ      (u16)(65536UL - (MAIN_Fosc*ʱʱs) / (TIM_ClkSource * (TIM_PS+1)))
 *         16λ              Xms:  (MAIN_Fosc / 1000) * X       Xus:  (MAIN_Fosc / 1000000) * X
 *      40,000,000L               1T:0-1ms    12T:0-19ms            1T:0-1638us    12T:0-19660us/
 *      35,000,000L               1T:0-1ms    12T:0-22ms            1T:0-1872us    12T:0-22469us/
 *      24,000,000L               1T:0-2ms    12T:0-32ms            1T:0-2739us    12T:0-32768us/
 *      22,118,400L               1T:0-2ms    12T:0-35ms            1T:0-2962us    12T:0-35555us/
 *      12,000,000L               1T:0-5ms    12T:0-65ms            1T:0-5461us    12T:0-65536us/
 *      11,059,200L               1T:0-5ms    12T:0-71ms            1T:0-5925us    12T:0-71111us/
 */
void Timer0_Init(u8 Mode, bit ClkSource, u16 TIM_Value, bit MsUs)
{
    u16 Value = 0;
    TM0PS = 0;      // 8λԤƵ(n+1), 0~255      TIM_PS = TM0PS+1  
    Value = (u16)(65536UL - ((MAIN_Fosc / (MsUs==0 ? 1000:1000000) /*ms:1000;us:1000000*/) * TIM_Value /*X*/) / (ClkSource==0 ? 12:1) /*12T;1T*/); // װسֵ

    T0_CT = 0;                      // 0: ʱ 1:   (ֻʱ)
    T0x12 = ClkSource;              // ʱԴ, 0: 12T, 1: 1T    (12TģʽԶʱ)
    TMOD = (TMOD & ~0x03) | Mode;   // ģʽ,0: 16λԶװ, 1: 16λʱ/, 2: 8λԶװ, 3: 16λԶװ
    TH0 = (Value) / 256;
    TL0 = (Value) % 256;
    // TF0 = 0;				        // TF0־, ʱ1
    TR0 = 1;				        // ʱ0ʼʱ
    ET0 = 1;				        // ʹܶʱ0ж
}   

void Timer0_Isr(void) interrupt TMR0_VECTOR
{
    Tcount++;
    B_1T = 1;
    B_10T = (Tcount % 10 == 0) ? 1 : 0;
    B_100T = (Tcount % 100 == 0) ? 1 : 0;
    B_1000T = (Tcount % 1000 == 0) ? 1 : 0;
    if (Tcount >= 5000)
        Tcount -= 5000;
}

